__all__ = ["Gaia"]

from dataclasses import dataclass
from typing import Any

from beet import Context, DensityFunction

from .density_functions import DensityFunction as DF


@dataclass
class Gaia:
    ctx: Context

    def df(self, id: str, function: DF):
        content: Any = function.generate()
        self.ctx.data[id] = DensityFunction(content)
        return DF.ref(id)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gaia_beet']

package_data = \
{'': ['*']}

install_requires = \
['beet>=0.45.3']

setup_kwargs = {
    'name': 'gaia-beet',
    'version': '0.2.1',
    'description': 'Beet plugin to generate Minecraft worldgen files',
    'long_description': '# gaia-beet\n\n[![GitHub Actions](https://github.com/misode/gaia/workflows/CI/badge.svg)](https://github.com/misode/gaia-beet/actions)\n[![PyPI](https://img.shields.io/pypi/v/gaia-beet.svg)](https://pypi.org/project/gaia-beet/)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/gaia-beet.svg)](https://pypi.org/project/gaia-beet/)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n[![Discord](https://img.shields.io/discord/900530660677156924?color=7289DA&label=discord&logo=discord&logoColor=fff)](https://discord.gg/98MdSGMm8j)\n\n> Beet plugin to generate Minecraft worldgen files\n\n## Introduction\n\nWriting density functions in JSON by hand can be tiring and confusing. This package allows you to write them as natural looking expressions.\n\n```py\ngaia.df("basic:foo", abs(DF.const(4) ** 3) + DF.ref("basic:bar"))\n```\n\n## Installation\n\nThe package can be installed with `pip`.\n\n```bash\n$ pip install gaia-beet\n```\n\n## Getting started\n\nWhen using with [`beet`](https://github.com/mcbeet/beet), a simple `beet.yml` is enough:\n```yml\npipeline:\n  - main\n```\n\nThis references a `main.py` plugin file where the density functions will be defined:\n```py\nfrom beet import Context\nfrom gaia_beet import Gaia, DensityFunction as DF\n\ndef beet_default(ctx: Context):\n    gaia = ctx.inject(Gaia)\n\n    blah = DF.slide(DF.const(2))\n\n    foo = gaia.df("basic:foo", abs(DF.const(4) ** 3) + blah)\n\n    gaia.df("basic:bar", blah * foo)\n```\n\n## Contributing\n\nContributions are welcome. Make sure to first open an issue discussing the problem or the new feature before creating a pull request. The project uses [`poetry`](https://python-poetry.org).\n\n```bash\n$ poetry install\n```\n\nYou can run the tests with `poetry run pytest`.\n\n```bash\n$ poetry run pytest\n```\n\nThe project must type-check with [`pyright`](https://github.com/microsoft/pyright). If you\'re using VSCode the [`pylance`](https://marketplace.visualstudio.com/items?itemName=ms-python.vscode-pylance) extension should report diagnostics automatically. You can also install the type-checker locally with `npm install` and run it from the command-line.\n\n```bash\n$ npm run watch\n$ npm run check\n```\n\nThe code follows the [`black`](https://github.com/psf/black) code style. Import statements are sorted with [`isort`](https://pycqa.github.io/isort/).\n\n```bash\n$ poetry run isort gaia_beet examples tests\n$ poetry run black gaia_beet examples tests\n$ poetry run black --check gaia_beet examples tests\n```\n\n---\n\nLicense - [MIT](https://github.com/misode/gaia-beet/blob/main/LICENSE)\n',
    'author': 'Misode',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/misode/gaia-beet',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



__ALL__ = ["DensityFunction"]

from dataclasses import dataclass
from typing import Any, Dict, Literal

GeneratedDensityFunction = float | str | Dict[str, Any]


class DensityFunction:
    def generate(self) -> GeneratedDensityFunction:
        raise NotImplementedError("Density function generate not implemented")

    def __add__(self, other: "DensityFunction"):
        return TwoArgumentsDF("add", self, other)

    def __mul__(self, other: "DensityFunction"):
        return TwoArgumentsDF("mul", self, other)

    def __pow__(self, exp: Literal[2] | Literal[3]):
        if exp == 2:
            return OneArgumentDF("square", self)
        elif exp == 3:
            return OneArgumentDF("cube", self)

    def __abs__(self):
        return OneArgumentDF("abs", self)

    @staticmethod
    def const(value: float):
        return ConstantDF(value)

    @staticmethod
    def ref(id: str):
        return ReferenceDF(id)

    @staticmethod
    def half_negative(argument: "DensityFunction"):
        return OneArgumentDF("half_negative", argument)

    @staticmethod
    def quarter_negative(argument: "DensityFunction"):
        return OneArgumentDF("quarter_negative", argument)

    @staticmethod
    def squeeze(argument: "DensityFunction"):
        return OneArgumentDF("squeeze", argument)

    @staticmethod
    def blend_density(argument: "DensityFunction"):
        return OneArgumentDF("blend_density", argument)

    @staticmethod
    def cache_2d(argument: "DensityFunction"):
        return OneArgumentDF("cache_2d", argument)

    @staticmethod
    def cache_all_in_cell(argument: "DensityFunction"):
        return OneArgumentDF("cache_all_in_cell", argument)

    @staticmethod
    def cache_once(argument: "DensityFunction"):
        return OneArgumentDF("cache_once", argument)

    @staticmethod
    def flat_cache(argument: "DensityFunction"):
        return OneArgumentDF("flat_cache", argument)

    @staticmethod
    def interpolated(argument: "DensityFunction"):
        return OneArgumentDF("interpolated", argument)

    @staticmethod
    def shift(argument: "DensityFunction"):
        return OneArgumentDF("shift", argument)

    @staticmethod
    def shift_a(argument: "DensityFunction"):
        return OneArgumentDF("shift_a", argument)

    @staticmethod
    def shift_b(argument: "DensityFunction"):
        return OneArgumentDF("shift_b", argument)

    @staticmethod
    def slide(argument: "DensityFunction"):
        return OneArgumentDF("slide", argument)

    @staticmethod
    def min(*arguments: "DensityFunction") -> "DensityFunction":
        result = arguments[-1]
        for a in arguments[-2::-1]:
            result = TwoArgumentsDF("min", a, result)
        return result

    @staticmethod
    def max(*arguments: "DensityFunction") -> "DensityFunction":
        result = arguments[-1]
        for a in arguments[-2::-1]:
            result = TwoArgumentsDF("max", a, result)
        return result


@dataclass
class ConstantDF(DensityFunction):
    value: float

    def generate(self):
        return self.value


@dataclass
class ReferenceDF(DensityFunction):
    id: str

    def generate(self):
        return self.id


@dataclass
class OneArgumentDF(DensityFunction):
    type: str
    argument: DensityFunction

    def generate(self):
        return {
            "type": f"minecraft:{self.type}",
            "argument": self.argument.generate(),
        }


@dataclass
class TwoArgumentsDF(DensityFunction):
    type: str
    argument1: DensityFunction
    argument2: DensityFunction

    def generate(self):
        return {
            "type": f"minecraft:{self.type}",
            "argument1": self.argument1.generate(),
            "argument2": self.argument2.generate(),
        }
